@@ -66,8 +66,18 @@ pub struct Config {
     #[serde(default)]
     #[serde(skip_serializing_if = "FutureIncompatReportConfig::is_none")]
     pub future_incompat_report: FutureIncompatReportConfig,
-    // TODO: cargo-new
-    // TODO: http
+    /// The `[cargo-new]` table.
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#cargo-new)
+    #[serde(default)]
+    #[serde(skip_serializing_if = "CargoNewConfig::is_none")]
+    pub cargo_new: CargoNewConfig,
+    /// The `[http]` table.
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#http)
+    #[serde(default)]
+    #[serde(skip_serializing_if = "HttpConfig::is_none")]
+    pub http: HttpConfig,
     // TODO: install
     /// The `[net]` table.
     ///
@@ -113,7 +123,7 @@ impl Config {
     /// Read config files hierarchically from the given directory and merges them.
     pub fn load_with_cwd<P: AsRef<Path>>(cwd: P) -> Result<Self> {
         let cwd = cwd.as_ref();
-        Self::_load_with_options(cwd, walk::cargo_home_with_cwd(cwd))
+        Self::_load_with_options(cwd, walk::cargo_home_with_cwd(cwd).as_deref())
     }
 
     /// Read config files hierarchically from the given directory and merges them.
@@ -121,14 +131,14 @@ impl Config {
         cwd: P,
         cargo_home: Q,
     ) -> Result<Self> {
-        Self::_load_with_options(cwd.as_ref(), cargo_home.into())
+        Self::_load_with_options(cwd.as_ref(), cargo_home.into().as_deref())
     }
     pub(crate) fn _load_with_options(
         current_dir: &Path,
-        cargo_home: Option<PathBuf>,
+        cargo_home: Option<&Path>,
     ) -> Result<Config> {
         let mut base = None;
-        for path in crate::Walk::with_cargo_home(current_dir, cargo_home) {
+        for path in crate::walk::WalkInner::with_cargo_home(current_dir, cargo_home) {
             let config = Self::_load_file(&path)?;
             match &mut base {
                 None => base = Some((path, config)),
@@ -168,8 +178,8 @@ impl Config {
     ///
     /// If `force` is `true`, this matches the way cargo's `--config` CLI option
     /// overrides config.
-    pub(crate) fn merge(&mut self, from: Self, force: bool) -> Result<()> {
-        crate::merge::Merge::merge(self, from, force)
+    pub(crate) fn merge(&mut self, low: Self, force: bool) -> Result<()> {
+        crate::merge::Merge::merge(self, low, force)
     }
 
     pub(crate) fn set_path(&mut self, path: &Path) {
@@ -181,6 +191,8 @@ impl Config {
         target_configs: &BTreeMap<String, TargetConfig>,
         override_target_rustflags: bool,
         build_rustflags: &Option<Flags>,
+        override_target_rustdocflags: bool,
+        build_rustdocflags: &Option<Flags>,
         target_triple: &TargetTripleRef<'_>,
         build_config: &easy::BuildConfig,
     ) -> Result<Option<TargetConfig>> {
@@ -195,6 +207,8 @@ impl Config {
         let mut target_runner = target_config.as_mut().and_then(|c| c.runner.take());
         let mut target_rustflags: Option<Flags> =
             target_config.as_mut().and_then(|c| c.rustflags.take());
+        let mut target_rustdocflags: Option<Flags> =
+            target_config.as_mut().and_then(|c| c.rustdocflags.take());
         if let Some(linker) = cx.env_dyn(&format!("CARGO_TARGET_{target_u_upper}_LINKER"))? {
             target_linker = Some(linker);
         }
@@ -214,6 +228,18 @@ impl Config {
                 target_rustflags @ None => *target_rustflags = Some(rustflags),
             }
         }
+        if let Some(rustdocflags) =
+            cx.env_dyn(&format!("CARGO_TARGET_{target_u_upper}_RUSTDOCFLAGS"))?
+        {
+            let mut rustdocflags =
+                Flags::from_space_separated(&rustdocflags.val, rustdocflags.definition.as_ref());
+            match &mut target_rustdocflags {
+                Some(target_rustdocflags) => {
+                    target_rustdocflags.flags.append(&mut rustdocflags.flags);
+                }
+                target_rustdocflags @ None => *target_rustdocflags = Some(rustdocflags),
+            }
+        }
         for (k, v) in target_configs {
             if !k.starts_with("cfg(") {
                 continue;
@@ -255,13 +281,31 @@ impl Config {
             target_config.get_or_insert_with(TargetConfig::default).runner = Some(runner);
         }
         if override_target_rustflags {
-            target_config.get_or_insert_with(TargetConfig::default).rustflags =
-                build_rustflags.clone();
+            target_config
+                .get_or_insert_with(TargetConfig::default)
+                .rustflags
+                .clone_from(build_rustflags);
         } else if let Some(rustflags) = target_rustflags {
             target_config.get_or_insert_with(TargetConfig::default).rustflags = Some(rustflags);
         } else {
-            target_config.get_or_insert_with(TargetConfig::default).rustflags =
-                build_rustflags.clone();
+            target_config
+                .get_or_insert_with(TargetConfig::default)
+                .rustflags
+                .clone_from(build_rustflags);
+        }
+        if override_target_rustdocflags {
+            target_config
+                .get_or_insert_with(TargetConfig::default)
+                .rustdocflags
+                .clone_from(build_rustdocflags);
+        } else if let Some(rustdocflags) = target_rustdocflags {
+            target_config.get_or_insert_with(TargetConfig::default).rustdocflags =
+                Some(rustdocflags);
+        } else {
+            target_config
+                .get_or_insert_with(TargetConfig::default)
+                .rustdocflags
+                .clone_from(build_rustdocflags);
         }
         Ok(target_config)
     }
@@ -338,9 +382,11 @@ pub struct BuildConfig {
     // Resolve contexts. Completely ignored in serialization and deserialization.
     #[serde(skip)]
     pub(crate) override_target_rustflags: bool,
+    #[serde(skip)]
+    pub(crate) override_target_rustdocflags: bool,
 }
 
-// https://github.com/rust-lang/cargo/blob/0.67.0/src/cargo/util/config/target.rs
+// https://github.com/rust-lang/cargo/blob/0.80.0/src/cargo/util/context/target.rs
 /// A `[target.<triple>]` or `[target.<cfg>]` table.
 ///
 /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#target)
@@ -361,6 +407,11 @@ pub struct TargetConfig {
     /// [reference (`target.<cfg>.rustflags`)](https://doc.rust-lang.org/nightly/cargo/reference/config.html#targetcfgrustflags)
     #[serde(skip_serializing_if = "Option::is_none")]
     pub rustflags: Option<Flags>,
+    /// [reference (`target.<triple>.rustdocflags`)](https://doc.rust-lang.org/nightly/cargo/reference/config.html#targettriplerustdocflags)
+    ///
+    /// [reference (`target.<cfg>.rustdocflags`)](https://doc.rust-lang.org/nightly/cargo/reference/config.html#targetcfgrustdocflags)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub rustdocflags: Option<Flags>,
     // TODO: links: https://doc.rust-lang.org/nightly/cargo/reference/config.html#targettriplelinks
 }
 
@@ -434,6 +485,126 @@ pub struct FutureIncompatReportConfig {
     pub frequency: Option<Value<Frequency>>,
 }
 
+/// The `[cargo-new]` table.
+///
+/// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#cargo-new)
+#[derive(Debug, Clone, Default, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[non_exhaustive]
+pub struct CargoNewConfig {
+    /// Specifies the source control system to use for initializing a new repository.
+    /// Valid values are git, hg (for Mercurial), pijul, fossil or none to disable this behavior.
+    /// Defaults to git, or none if already inside a VCS repository. Can be overridden with the --vcs CLI option.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub vcs: Option<Value<VersionControlSoftware>>,
+}
+
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[non_exhaustive]
+pub enum VersionControlSoftware {
+    /// Git
+    Git,
+    /// Mercurial
+    #[serde(rename = "hg")]
+    Mercurial,
+    /// Pijul
+    Pijul,
+    /// Fossil
+    Fossil,
+    /// No VCS
+    None,
+}
+
+impl VersionControlSoftware {
+    pub const fn as_str(self) -> &'static str {
+        match self {
+            VersionControlSoftware::Git => "git",
+            VersionControlSoftware::Mercurial => "hg",
+            VersionControlSoftware::Pijul => "pijul",
+            VersionControlSoftware::Fossil => "fossil",
+            VersionControlSoftware::None => "none",
+        }
+    }
+}
+
+impl FromStr for VersionControlSoftware {
+    type Err = Error;
+
+    fn from_str(vcs: &str) -> Result<Self, Self::Err> {
+        match vcs {
+            "git" => Ok(Self::Git),
+            "hg" => Ok(Self::Mercurial),
+            "pijul" => Ok(Self::Pijul),
+            "fossil" => Ok(Self::Fossil),
+            "none" => Ok(Self::None),
+            other => bail!("must be git, hg, pijul, fossil, none, but found `{other}`"),
+        }
+    }
+}
+
+/// The `[http]` table.
+///
+/// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#http)
+#[derive(Debug, Clone, Default, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+#[non_exhaustive]
+pub struct HttpConfig {
+    /// If true, enables debugging of HTTP requests.
+    /// The debug information can be seen by setting the `CARGO_LOG=network=debug` environment variable
+    /// (or use `network=trace` for even more information).
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#httpdebug)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub debug: Option<Value<bool>>,
+    /// Sets an HTTP and HTTPS proxy to use. The format is in libcurl format as in `[protocol://]host[:port]`.
+    /// If not set, Cargo will also check the http.proxy setting in your global git configuration.
+    /// If none of those are set, the HTTPS_PROXY or https_proxy environment variables set the proxy for HTTPS requests,
+    /// and http_proxy sets it for HTTP requests.
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#httpproxy)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub proxy: Option<Value<String>>,
+    /// Sets the timeout for each HTTP request, in seconds.
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#httptimeout)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub timeout: Option<Value<u32>>,
+    /// Path to a Certificate Authority (CA) bundle file, used to verify TLS certificates.
+    /// If not specified, Cargo attempts to use the system certificates.
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#httpcainfo)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub cainfo: Option<Value<String>>,
+    /// This determines whether or not TLS certificate revocation checks should be performed.
+    /// This only works on Windows.
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#httpcheck-revoke)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub check_revoke: Option<Value<bool>>,
+    // TODO: handle ssl-version
+    /// This setting controls timeout behavior for slow connections.
+    /// If the average transfer speed in bytes per second is below the given value
+    /// for `http.timeout` seconds (default 30 seconds), then the connection is considered too slow and Cargo will abort and retry.
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#httplow-speed-limit)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub low_speed_limit: Option<Value<u32>>,
+    /// When true, Cargo will attempt to use the HTTP2 protocol with multiplexing.
+    /// This allows multiple requests to use the same connection, usually improving performance when fetching multiple files.
+    /// If false, Cargo will use HTTP 1.1 without pipelining.
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#httpmultiplexing)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub multiplexing: Option<Value<bool>>,
+    /// Specifies a custom user-agent header to use.
+    /// The default if not specified is a string that includes Cargo’s version.
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#httpuser-agent)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub user_agent: Option<Value<String>>,
+}
+
 /// The `[net]` table.
 ///
 /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#net)
@@ -772,6 +943,7 @@ impl Flags {
     /// - `CARGO_TARGET_<triple>_RUSTFLAGS`
     /// - `CARGO_BUILD_RUSTFLAGS`
     /// - `RUSTDOCFLAGS`
+    /// - `CARGO_TARGET_<triple>_RUSTDOCFLAGS`
     /// - `CARGO_BUILD_RUSTDOCFLAGS`
     ///
     /// And the following configs:
@@ -779,6 +951,7 @@ impl Flags {
     /// - `target.<triple>.rustflags`
     /// - `target.<cfg>.rustflags`
     /// - `build.rustflags`
+    /// - `target.<triple>.rustdocflags` (Cargo 1.78+)
     /// - `build.rustdocflags`
     ///
     /// See also `encode_space_separated`.
@@ -811,7 +984,7 @@ impl<'de> Deserialize<'de> for Flags {
     }
 }
 
-// https://github.com/rust-lang/cargo/blob/0.67.0/src/cargo/util/config/path.rs
+// https://github.com/rust-lang/cargo/blob/0.80.0/src/cargo/util/context/path.rs
 #[derive(Debug, Deserialize, PartialEq, Clone)]
 #[serde(transparent)]
 pub struct ConfigRelativePath(pub(crate) Value<String>);
@@ -1053,5 +1226,7 @@ pub(crate) fn split_encoded(s: &str) -> impl Iterator<Item = &str> {
     s.split('\x1f')
 }
 pub(crate) fn split_space_separated(s: &str) -> impl Iterator<Item = &str> {
+    // TODO: tab/line?
+    // https://github.com/rust-lang/cargo/blob/0.80.0/src/cargo/util/context/path.rs#L89
     s.split(' ').map(str::trim).filter(|s| !s.is_empty())
 }
