@@ -13,10 +13,16 @@ use serde::ser::{Serialize, Serializer};
 use serde_derive::Serialize;
 
 use crate::{
-    de::{self, split_encoded, split_space_separated, Color, Frequency, RegistriesProtocol, When},
+    de::{
+        self, split_encoded, split_space_separated, Color, Frequency, RegistriesProtocol,
+        VersionControlSoftware, When,
+    },
     error::{Context as _, Result},
     process::ProcessBuilder,
-    resolve::{ResolveContext, ResolveOptions, TargetTriple, TargetTripleBorrow, TargetTripleRef},
+    resolve::{
+        CargoVersion, ResolveContext, ResolveOptions, RustcVersion, TargetTriple,
+        TargetTripleBorrow, TargetTripleRef,
+    },
     value::Value,
 };
 
@@ -56,8 +62,18 @@ pub struct Config {
     #[serde(default)]
     #[serde(skip_serializing_if = "FutureIncompatReportConfig::is_none")]
     pub future_incompat_report: FutureIncompatReportConfig,
-    // TODO: cargo-new
-    // TODO: http
+    /// The `[cargo-new]` table.
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#cargo-new)
+    #[serde(default)]
+    #[serde(skip_serializing_if = "CargoNewConfig::is_none")]
+    pub cargo_new: CargoNewConfig,
+    /// The `[http]` table.
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#http)
+    #[serde(default)]
+    #[serde(skip_serializing_if = "HttpConfig::is_none")]
+    pub http: HttpConfig,
     // TODO: install
     /// The `[net]` table.
     ///
@@ -123,11 +139,11 @@ impl Config {
         let cwd = cwd.as_ref();
         let cx = options.into_context(cwd.to_owned());
 
-        let de = de::Config::_load_with_options(&cx.current_dir, cx.cargo_home(cwd).clone())?;
+        let de = de::Config::_load_with_options(&cx.current_dir, cx.cargo_home(cwd).as_deref())?;
         Self::from_unresolved(de, cx)
     }
 
-    pub(crate) fn from_unresolved(mut de: de::Config, cx: ResolveContext) -> Result<Self> {
+    fn from_unresolved(mut de: de::Config, cx: ResolveContext) -> Result<Self> {
         de.apply_env(&cx)?;
 
         let mut alias = BTreeMap::new();
@@ -142,6 +158,8 @@ impl Config {
         }
         let future_incompat_report =
             FutureIncompatReportConfig::from_unresolved(de.future_incompat_report);
+        let cargo_new = CargoNewConfig::from_unresolved(de.cargo_new);
+        let http = HttpConfig::from_unresolved(de.http);
         let net = NetConfig::from_unresolved(de.net);
         let mut registries = BTreeMap::new();
         for (k, v) in de.registries {
@@ -156,6 +174,8 @@ impl Config {
             doc,
             env,
             future_incompat_report,
+            cargo_new,
+            http,
             net,
             registries,
             registry,
@@ -174,7 +194,7 @@ impl Config {
     /// 1. `--target` option (`targets`)
     /// 2. `CARGO_BUILD_TARGET` environment variable
     /// 3. `build.target` config
-    /// 4. host triple (`host`)
+    /// 4. [host triple](Self::host_triple)
     ///
     /// **Note:** The result of this function is intended to handle target-specific
     /// configurations and is not always appropriate to propagate directly to Cargo.
@@ -201,7 +221,6 @@ impl Config {
     /// With single-target:
     ///
     /// ```no_run
-    /// # fn main() -> anyhow::Result<()> {
     /// use anyhow::bail;
     /// use clap::Parser;
     ///
@@ -221,13 +240,12 @@ impl Config {
     /// let target = targets.pop().unwrap();
     ///
     /// println!("{:?}", config.rustflags(target));
-    /// # Ok(()) }
+    /// # Ok::<(), anyhow::Error>(())
     /// ```
     ///
     /// With multi-target:
     ///
     /// ```no_run
-    /// # fn main() -> anyhow::Result<()> {
     /// use clap::Parser;
     ///
     /// #[derive(Parser)]
@@ -244,7 +262,7 @@ impl Config {
     /// for target in targets {
     ///     println!("{:?}", config.rustflags(target)?);
     /// }
-    /// # Ok(()) }
+    /// # Ok::<(), anyhow::Error>(())
     /// ```
     pub fn build_target_for_config<'a, I: IntoIterator<Item = T>, T: Into<TargetTripleRef<'a>>>(
         &self,
@@ -301,6 +319,8 @@ impl Config {
                     &self.de_target,
                     self.build.override_target_rustflags,
                     &self.build.de_rustflags,
+                    self.build.override_target_rustdocflags,
+                    &self.build.de_rustdocflags,
                     target,
                     &self.build,
                 )?
@@ -338,6 +358,15 @@ impl Config {
         self.init_target_config(&target)?;
         Ok(self.target.borrow()[target.cli_target()].rustflags.clone())
     }
+    /// Returns the resolved rustdocflags for the given target.
+    pub fn rustdocflags<'a, T: Into<TargetTripleRef<'a>>>(
+        &self,
+        target: T,
+    ) -> Result<Option<Flags>> {
+        let target = target.into();
+        self.init_target_config(&target)?;
+        Ok(self.target.borrow()[target.cli_target()].rustdocflags.clone())
+    }
 
     /// Returns the path and args that calls `rustc`.
     ///
@@ -346,7 +375,7 @@ impl Config {
     /// the path is the wrapper path and the argument is the rustc path.
     /// Otherwise, the path is the rustc path.
     ///
-    /// If you set `rustc` path by [`ResolveOptions::rustc`], this returns the path set by that method.
+    /// If you set `rustc` path by [`ResolveOptions::rustc`], this returns the path set by it.
     pub fn rustc(&self) -> &PathAndArgs {
         self.cx.rustc(&self.build)
     }
@@ -354,7 +383,7 @@ impl Config {
     ///
     /// The returned path is the value of the `CARGO` environment variable if it is set. Otherwise, "cargo".
     ///
-    /// If you set `cargo` path by [`ResolveOptions::cargo`], this returns the path set by that method.
+    /// If you set `cargo` path by [`ResolveOptions::cargo`], this returns the path set by it.
     pub fn cargo(&self) -> &OsStr {
         &self.cx.cargo
     }
@@ -362,6 +391,29 @@ impl Config {
     pub fn host_triple(&self) -> Result<&str> {
         self.cx.host_triple(&self.build)
     }
+    /// Returns the version of the [current rustc](Self::rustc).
+    ///
+    /// The result is usually the same as [`cargo_version`](Self::cargo_version),
+    /// but it may differ if a different rustc is specified in config or if the
+    /// [user is manipulating the output of the rustc](https://github.com/taiki-e/cargo-minimal-versions/issues/29).
+    ///
+    /// # rustc_version vs cargo_version
+    ///
+    /// Which is the preferred to use depends on the situation:
+    ///
+    /// - You will need to know the **rustc** version to determine whether options passed to rustc
+    ///   via RUSTFLAGS or RUSTDOCFLAGS like `-C instrument-coverage` are available.
+    /// - You will need to know the **cargo** version to determine whether fields in `Cargo.toml`
+    ///   or cargo’s CLI options are available.
+    pub fn rustc_version(&self) -> Result<RustcVersion> {
+        self.cx.rustc_version(&self.build)
+    }
+    /// Returns the version of the [current cargo](Self::cargo).
+    ///
+    /// See also [`rustc_version`](Self::rustc_version).
+    pub fn cargo_version(&self) -> Result<CargoVersion> {
+        self.cx.cargo_version(&self.build)
+    }
 
     // TODO: add override instead?
     // /// Merges the given config into this config.
@@ -371,8 +423,8 @@ impl Config {
     // ///
     // /// If `force` is `true`, this matches the way cargo's `--config` CLI option
     // /// overrides config.
-    // pub fn merge(&mut self, from: Self, force: bool) -> Result<()> {
-    //     merge::Merge::merge(self, from, force)
+    // pub fn merge(&mut self, low: Self, force: bool) -> Result<()> {
+    //     merge::Merge::merge(self, low, force)
     // }
 }
 
@@ -447,6 +499,10 @@ pub struct BuildConfig {
     /// of strings or a space-separated string.
     ///
     /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#buildrustdocflags)
+    ///
+    /// **Note:** This field does not reflect the target-specific rustdocflags
+    /// configuration, so you may want to use [`Config::rustdocflags`] which respects
+    /// the target-specific rustdocflags configuration.
     #[serde(skip_serializing_if = "Option::is_none")]
     pub rustdocflags: Option<Flags>,
     /// Whether or not to perform incremental compilation.
@@ -464,7 +520,11 @@ pub struct BuildConfig {
     #[serde(skip)]
     override_target_rustflags: bool,
     #[serde(skip)]
-    pub(crate) de_rustflags: Option<de::Flags>,
+    de_rustflags: Option<de::Flags>,
+    #[serde(skip)]
+    override_target_rustdocflags: bool,
+    #[serde(skip)]
+    de_rustdocflags: Option<de::Flags>,
 }
 
 impl BuildConfig {
@@ -492,12 +552,14 @@ impl BuildConfig {
         let de_rustflags = de.rustflags.clone();
         let rustflags =
             de.rustflags.map(|v| Flags { flags: v.flags.into_iter().map(|v| v.val).collect() });
+        let de_rustdocflags = de.rustdocflags.clone();
         let rustdocflags =
             de.rustdocflags.map(|v| Flags { flags: v.flags.into_iter().map(|v| v.val).collect() });
         let incremental = de.incremental.map(|v| v.val);
         let dep_info_basedir =
             de.dep_info_basedir.map(|v| v.resolve_as_path(current_dir).into_owned());
         let override_target_rustflags = de.override_target_rustflags;
+        let override_target_rustdocflags = de.override_target_rustdocflags;
         Self {
             jobs,
             rustc,
@@ -512,11 +574,13 @@ impl BuildConfig {
             dep_info_basedir,
             override_target_rustflags,
             de_rustflags,
+            override_target_rustdocflags,
+            de_rustdocflags,
         }
     }
 }
 
-// https://github.com/rust-lang/cargo/blob/0.67.0/src/cargo/util/config/target.rs
+// https://github.com/rust-lang/cargo/blob/0.80.0/src/cargo/util/context/target.rs
 /// A `[target.<triple>]` or `[target.<cfg>]` table.
 ///
 /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#target)
@@ -537,11 +601,16 @@ pub struct TargetConfig {
     /// [reference (`target.<cfg>.rustflags`)](https://doc.rust-lang.org/nightly/cargo/reference/config.html#targetcfgrustflags)
     #[serde(skip_serializing_if = "Option::is_none")]
     pub rustflags: Option<Flags>,
+    /// [reference (`target.<triple>.rustdocflags`)](https://doc.rust-lang.org/nightly/cargo/reference/config.html#targettriplerustdocflags)
+    ///
+    /// [reference (`target.<cfg>.rustdocflags`)](https://doc.rust-lang.org/nightly/cargo/reference/config.html#targetcfgrustdocflags)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub rustdocflags: Option<Flags>,
     // TODO: links: https://doc.rust-lang.org/nightly/cargo/reference/config.html#targettriplelinks
 }
 
 impl TargetConfig {
-    pub(crate) fn from_unresolved(de: de::TargetConfig, current_dir: &Path) -> Self {
+    fn from_unresolved(de: de::TargetConfig, current_dir: &Path) -> Self {
         let linker = de.linker.map(|v| v.resolve_as_program_path(current_dir).into_owned());
         let runner = match de.runner {
             Some(v) => Some(PathAndArgs {
@@ -552,7 +621,9 @@ impl TargetConfig {
         };
         let rustflags =
             de.rustflags.map(|v| Flags { flags: v.flags.into_iter().map(|v| v.val).collect() });
-        Self { linker, runner, rustflags }
+        let rustdocflags =
+            de.rustdocflags.map(|v| Flags { flags: v.flags.into_iter().map(|v| v.val).collect() });
+        Self { linker, runner, rustflags, rustdocflags }
     }
 }
 
@@ -572,7 +643,7 @@ pub struct DocConfig {
 }
 
 impl DocConfig {
-    pub(crate) fn from_unresolved(de: de::DocConfig, current_dir: &Path) -> Self {
+    fn from_unresolved(de: de::DocConfig, current_dir: &Path) -> Self {
         let browser = de.browser.map(|v| PathAndArgs {
             path: v.path.resolve_program(current_dir).into_owned(),
             args: v.args.into_iter().map(|v| v.val.into()).collect(),
@@ -593,7 +664,7 @@ pub struct EnvConfigValue {
 }
 
 impl EnvConfigValue {
-    pub(crate) fn from_unresolved(de: de::EnvConfigValue, current_dir: &Path) -> Self {
+    fn from_unresolved(de: de::EnvConfigValue, current_dir: &Path) -> Self {
         if let de::EnvConfigValue::Table {
             force, relative: Some(Value { val: true, .. }), ..
         } = &de
@@ -664,12 +735,109 @@ pub struct FutureIncompatReportConfig {
 }
 
 impl FutureIncompatReportConfig {
-    pub(crate) fn from_unresolved(de: de::FutureIncompatReportConfig) -> Self {
+    fn from_unresolved(de: de::FutureIncompatReportConfig) -> Self {
         let frequency = de.frequency.map(|v| v.val);
         Self { frequency }
     }
 }
 
+/// The `[cargo-new]` table.
+///
+/// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#cargo-new)
+#[derive(Debug, Clone, Default, Serialize)]
+#[serde(rename_all = "kebab-case")]
+#[non_exhaustive]
+pub struct CargoNewConfig {
+    /// Specifies the source control system to use for initializing a new repository.
+    /// Valid values are git, hg (for Mercurial), pijul, fossil or none to disable this behavior.
+    /// Defaults to git, or none if already inside a VCS repository. Can be overridden with the --vcs CLI option.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub vcs: Option<VersionControlSoftware>,
+}
+
+impl CargoNewConfig {
+    fn from_unresolved(de: de::CargoNewConfig) -> Self {
+        Self { vcs: de.vcs.map(|v| v.val) }
+    }
+}
+
+/// The `[http]` table.
+///
+/// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#http)
+#[derive(Debug, Clone, Default, Serialize)]
+#[serde(rename_all = "kebab-case")]
+#[non_exhaustive]
+pub struct HttpConfig {
+    /// If true, enables debugging of HTTP requests.
+    /// The debug information can be seen by setting the `CARGO_LOG=network=debug` environment variable
+    /// (or use `network=trace` for even more information).
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#httpdebug)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub debug: Option<bool>,
+    /// Sets an HTTP and HTTPS proxy to use. The format is in libcurl format as in `[protocol://]host[:port]`.
+    /// If not set, Cargo will also check the http.proxy setting in your global git configuration.
+    /// If none of those are set, the HTTPS_PROXY or https_proxy environment variables set the proxy for HTTPS requests,
+    /// and http_proxy sets it for HTTP requests.
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#httpproxy)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub proxy: Option<String>,
+    /// Sets the timeout for each HTTP request, in seconds.
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#httptimeout)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub timeout: Option<u32>,
+    /// Path to a Certificate Authority (CA) bundle file, used to verify TLS certificates.
+    /// If not specified, Cargo attempts to use the system certificates.
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#httpcainfo)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub cainfo: Option<String>,
+    /// This determines whether or not TLS certificate revocation checks should be performed.
+    /// This only works on Windows.
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#httpcheck-revoke)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub check_revoke: Option<bool>,
+    // TODO: Add ssl-version
+    /// This setting controls timeout behavior for slow connections.
+    /// If the average transfer speed in bytes per second is below the given value
+    /// for `http.timeout` seconds (default 30 seconds), then the connection is considered too slow and Cargo will abort and retry.
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#httplow-speed-limit)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub low_speed_limit: Option<u32>,
+    /// When true, Cargo will attempt to use the HTTP2 protocol with multiplexing.
+    /// This allows multiple requests to use the same connection, usually improving performance when fetching multiple files.
+    /// If false, Cargo will use HTTP 1.1 without pipelining.
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#httpmultiplexing)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub multiplexing: Option<bool>,
+    /// Specifies a custom user-agent header to use.
+    /// The default if not specified is a string that includes Cargo’s version.
+    ///
+    /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#httpuser-agent)
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub user_agent: Option<String>,
+}
+
+impl HttpConfig {
+    fn from_unresolved(de: de::HttpConfig) -> Self {
+        Self {
+            debug: de.debug.map(|v| v.val),
+            proxy: de.proxy.map(|v| v.val),
+            timeout: de.timeout.map(|v| v.val),
+            cainfo: de.cainfo.map(|v| v.val),
+            check_revoke: de.check_revoke.map(|v| v.val),
+            low_speed_limit: de.low_speed_limit.map(|v| v.val),
+            multiplexing: de.multiplexing.map(|v| v.val),
+            user_agent: de.user_agent.map(|v| v.val),
+        }
+    }
+}
+
 /// The `[net]` table.
 ///
 /// [reference](https://doc.rust-lang.org/nightly/cargo/reference/config.html#net)
@@ -700,7 +868,7 @@ pub struct NetConfig {
 }
 
 impl NetConfig {
-    pub(crate) fn from_unresolved(de: de::NetConfig) -> Self {
+    fn from_unresolved(de: de::NetConfig) -> Self {
         let retry = de.retry.map(|v| v.val);
         let git_fetch_with_cli = de.git_fetch_with_cli.map(|v| v.val);
         let offline = de.offline.map(|v| v.val);
@@ -738,7 +906,7 @@ pub struct RegistriesConfigValue {
 }
 
 impl RegistriesConfigValue {
-    pub(crate) fn from_unresolved(de: de::RegistriesConfigValue) -> Self {
+    fn from_unresolved(de: de::RegistriesConfigValue) -> Self {
         let index = de.index.map(|v| v.val);
         let token = de.token.map(|v| v.val);
         let protocol = de.protocol.map(|v| match v.val {
@@ -788,7 +956,7 @@ pub struct RegistryConfig {
 }
 
 impl RegistryConfig {
-    pub(crate) fn from_unresolved(de: de::RegistryConfig) -> Self {
+    fn from_unresolved(de: de::RegistryConfig) -> Self {
         let default = de.default.map(|v| v.val);
         let token = de.token.map(|v| v.val);
         Self { default, token }
@@ -834,7 +1002,7 @@ pub struct TermConfig {
 }
 
 impl TermConfig {
-    pub(crate) fn from_unresolved(de: de::TermConfig) -> Self {
+    fn from_unresolved(de: de::TermConfig) -> Self {
         let quiet = de.quiet.map(|v| v.val);
         let verbose = de.verbose.map(|v| v.val);
         let color = de.color.map(|v| v.val);
@@ -860,14 +1028,14 @@ pub struct TermProgressConfig {
 }
 
 impl TermProgressConfig {
-    pub(crate) fn from_unresolved(de: de::TermProgress) -> Self {
+    fn from_unresolved(de: de::TermProgress) -> Self {
         let when = de.when.map(|v| v.val);
         let width = de.width.map(|v| v.val);
         Self { when, width }
     }
 }
 
-/// A representation of rustflags and rustdocflags.
+/// A representation of rustflags or rustdocflags.
 #[derive(Debug, Clone, Default, PartialEq, Eq, Serialize)]
 #[serde(transparent)]
 #[non_exhaustive]
@@ -876,7 +1044,7 @@ pub struct Flags {
 }
 
 impl Flags {
-    /// Creates a rustflags from a string separated with ASCII unit separator ('\x1f').
+    /// Creates a rustflags or rustdocflags from a string separated with ASCII unit separator ('\x1f').
     ///
     /// This is a valid format for the following environment variables:
     ///
@@ -888,7 +1056,7 @@ impl Flags {
         Self { flags: split_encoded(s).map(str::to_owned).collect() }
     }
 
-    /// Creates a rustflags from a string separated with space (' ').
+    /// Creates a rustflags or rustdocflags from a string separated with space (' ').
     ///
     /// This is a valid format for the following environment variables:
     ///
@@ -896,6 +1064,7 @@ impl Flags {
     /// - `CARGO_TARGET_<triple>_RUSTFLAGS`
     /// - `CARGO_BUILD_RUSTFLAGS`
     /// - `RUSTDOCFLAGS`
+    /// - `CARGO_TARGET_<triple>_RUSTDOCFLAGS`
     /// - `CARGO_BUILD_RUSTDOCFLAGS`
     ///
     /// And the following configs:
@@ -903,6 +1072,7 @@ impl Flags {
     /// - `target.<triple>.rustflags`
     /// - `target.<cfg>.rustflags`
     /// - `build.rustflags`
+    /// - `target.<triple>.rustdocflags` (Cargo 1.78+)
     /// - `build.rustdocflags`
     ///
     /// See also [`encode_space_separated`](Self::encode_space_separated).
@@ -910,7 +1080,7 @@ impl Flags {
         Self { flags: split_space_separated(s).map(str::to_owned).collect() }
     }
 
-    /// Concatenates this rustflags with ASCII unit separator ('\x1f').
+    /// Concatenates this rustflags or rustdocflags with ASCII unit separator ('\x1f').
     ///
     /// This is a valid format for the following environment variables:
     ///
@@ -925,8 +1095,8 @@ impl Flags {
     /// separator, Cargo will interpret it as a separator.
     ///
     /// Since it is not easy to insert an ASCII unit separator in a toml file or
-    /// Shell environment variable, this usually occurs when this rustflags is
-    /// created in the wrong way ([`from_encoded`](Self::from_encoded) vs
+    /// Shell environment variable, this usually occurs when this rustflags or rustdocflags
+    /// is created in the wrong way ([`from_encoded`](Self::from_encoded) vs
     /// [`from_space_separated`](Self::from_space_separated)) or when a flag
     /// containing a separator is written in the rust code ([`push`](Self::push),
     /// `into`, `from`, etc.).
@@ -934,7 +1104,7 @@ impl Flags {
         self.encode_internal('\x1f')
     }
 
-    /// Concatenates this rustflags with space (' ').
+    /// Concatenates this rustflags or rustdocflags with space (' ').
     ///
     /// This is a valid format for the following environment variables:
     ///
@@ -942,6 +1112,7 @@ impl Flags {
     /// - `CARGO_TARGET_<triple>_RUSTFLAGS`
     /// - `CARGO_BUILD_RUSTFLAGS`
     /// - `RUSTDOCFLAGS`
+    /// - `CARGO_TARGET_<triple>_RUSTDOCFLAGS`
     /// - `CARGO_BUILD_RUSTDOCFLAGS`
     ///
     /// And the following configs:
@@ -949,6 +1120,7 @@ impl Flags {
     /// - `target.<triple>.rustflags`
     /// - `target.<cfg>.rustflags`
     /// - `build.rustflags`
+    /// - `target.<triple>.rustdocflags` (Cargo 1.78+)
     /// - `build.rustdocflags`
     ///
     /// # Errors
@@ -980,7 +1152,7 @@ impl Flags {
         Ok(buf)
     }
 
-    /// Appends a flag to the back of this rustflags.
+    /// Appends a flag to the back of this rustflags or rustdocflags.
     pub fn push<S: Into<String>>(&mut self, flag: S) {
         self.flags.push(flag.into());
     }
@@ -1055,13 +1227,18 @@ impl Serialize for PathAndArgs {
 
 impl From<PathAndArgs> for Command {
     fn from(value: PathAndArgs) -> Self {
-        let mut cmd = Command::new(value.path);
-        cmd.args(value.args);
+        Self::from(&value)
+    }
+}
+impl From<&PathAndArgs> for Command {
+    fn from(value: &PathAndArgs) -> Self {
+        let mut cmd = Command::new(&value.path);
+        cmd.args(&value.args);
         cmd
     }
 }
-impl From<PathAndArgs> for ProcessBuilder {
-    fn from(value: PathAndArgs) -> Self {
+impl From<&PathAndArgs> for ProcessBuilder {
+    fn from(value: &PathAndArgs) -> Self {
         ProcessBuilder::from_std(Command::from(value))
     }
 }
@@ -1075,13 +1252,13 @@ pub struct StringList {
 }
 
 impl StringList {
-    pub(crate) fn from_string(value: &str) -> Self {
+    fn from_string(value: &str) -> Self {
         Self { list: split_space_separated(value).map(str::to_owned).collect() }
     }
-    pub(crate) fn from_array(list: Vec<String>) -> Self {
+    fn from_array(list: Vec<String>) -> Self {
         Self { list }
     }
-    pub(crate) fn from_unresolved(value: de::StringList) -> Self {
+    fn from_unresolved(value: de::StringList) -> Self {
         Self { list: value.list.into_iter().map(|v| v.val).collect() }
     }
 }
