@@ -5,8 +5,9 @@
 
 use crate::{
     de::{
-        BuildConfig, Config, DocConfig, Flags, FutureIncompatReportConfig, NetConfig, PathAndArgs,
-        RegistriesConfigValue, RegistryConfig, StringList, StringOrArray, TermConfig, TermProgress,
+        BuildConfig, CargoNewConfig, Config, DocConfig, Flags, FutureIncompatReportConfig,
+        HttpConfig, NetConfig, PathAndArgs, RegistriesConfigValue, RegistryConfig, StringList,
+        StringOrArray, TermConfig, TermProgress,
     },
     error::{Context as _, Error, Result},
     resolve::ResolveContext,
@@ -96,6 +97,7 @@ impl Config {
         self.build.apply_env(cx)?;
         self.doc.apply_env(cx)?;
         self.future_incompat_report.apply_env(cx)?;
+        self.http.apply_env(cx)?;
         self.net.apply_env(cx)?;
         self.registry.apply_env(cx)?;
         self.term.apply_env(cx)?;
@@ -204,15 +206,20 @@ impl ApplyEnv for BuildConfig {
         }
         // 1. CARGO_ENCODED_RUSTDOCFLAGS
         // 2. RUSTDOCFLAGS
-        // 3. build.rustdocflags (CARGO_BUILD_RUSTDOCFLAGS)
+        // 3. target.<triple>.rustdocflags (CARGO_TARGET_<triple>_RUSTDOCFLAGS)
+        // 4. build.rustdocflags (CARGO_BUILD_RUSTDOCFLAGS)
+        // For 3, we handle it in de::Config::resolve_target.
         // https://doc.rust-lang.org/nightly/cargo/reference/config.html#buildrustdocflags
+        self.override_target_rustdocflags = false;
         if let Some(rustdocflags) = cx.env("CARGO_ENCODED_RUSTDOCFLAGS")? {
             self.rustdocflags = Some(Flags::from_encoded(&rustdocflags));
+            self.override_target_rustdocflags = true;
         } else if let Some(rustdocflags) = cx.env("RUSTDOCFLAGS")? {
             self.rustdocflags = Some(Flags::from_space_separated(
                 &rustdocflags.val,
                 rustdocflags.definition.as_ref(),
             ));
+            self.override_target_rustdocflags = true;
         } else if let Some(rustdocflags) = cx.env("CARGO_BUILD_RUSTDOCFLAGS")? {
             self.rustdocflags = Some(Flags::from_space_separated(
                 &rustdocflags.val,
@@ -245,7 +252,7 @@ impl ApplyEnv for BuildConfig {
 impl ApplyEnv for DocConfig {
     fn apply_env(&mut self, cx: &ResolveContext) -> Result<()> {
         // doc.browser config value is prefer over BROWSER environment variable.
-        // https://github.com/rust-lang/cargo/blob/0.67.0/src/cargo/ops/cargo_doc.rs#L52-L53
+        // https://github.com/rust-lang/cargo/blob/0.80.0/src/cargo/ops/cargo_doc.rs#L143-L144
         if self.browser.is_none() {
             if let Some(browser) = cx.env("BROWSER")? {
                 self.browser = Some(
@@ -268,6 +275,58 @@ impl ApplyEnv for FutureIncompatReportConfig {
     }
 }
 
+impl ApplyEnv for CargoNewConfig {
+    fn apply_env(&mut self, cx: &ResolveContext) -> Result<()> {
+        // https://doc.rust-lang.org/nightly/cargo/reference/config.html#cargo-newvcs
+        if let Some(vcs) = cx.env_parse("CARGO_CARGO_NEW_VCS")? {
+            self.vcs = Some(vcs);
+        }
+        Ok(())
+    }
+}
+
+impl ApplyEnv for HttpConfig {
+    fn apply_env(&mut self, cx: &ResolveContext) -> Result<()> {
+        // https://doc.rust-lang.org/nightly/cargo/reference/config.html#httpdebug
+        if let Some(debug) = cx.env_parse("CARGO_HTTP_DEBUG")? {
+            self.debug = Some(debug);
+        }
+        // https://doc.rust-lang.org/nightly/cargo/reference/config.html#httpproxy
+        // TODO:
+        // > CARGO_HTTP_PROXY or HTTPS_PROXY or https_proxy or http_proxy
+        if let Some(proxy) = cx.env("CARGO_HTTP_PROXY")? {
+            self.proxy = Some(proxy);
+        }
+        // https://doc.rust-lang.org/nightly/cargo/reference/config.html#httptimeout
+        // TODO:
+        // > CARGO_HTTP_TIMEOUT or HTTP_TIMEOUT
+        if let Some(timeout) = cx.env_parse("CARGO_HTTP_TIMEOUT")? {
+            self.timeout = Some(timeout);
+        }
+        // https://doc.rust-lang.org/nightly/cargo/reference/config.html#httpcainfo
+        if let Some(cainfo) = cx.env("CARGO_HTTP_CAINFO")? {
+            self.cainfo = Some(cainfo);
+        }
+        // https://doc.rust-lang.org/nightly/cargo/reference/config.html#httpcheck-revoke
+        if let Some(check_revoke) = cx.env_parse("CARGO_HTTP_CHECK_REVOKE")? {
+            self.check_revoke = Some(check_revoke);
+        }
+        // https://doc.rust-lang.org/nightly/cargo/reference/config.html#httplow-speed-limit
+        if let Some(low_speed_limit) = cx.env_parse("CARGO_HTTP_LOW_SPEED_LIMIT")? {
+            self.low_speed_limit = Some(low_speed_limit);
+        }
+        // https://doc.rust-lang.org/nightly/cargo/reference/config.html#httpmultiplexing
+        if let Some(multiplexing) = cx.env_parse("CARGO_HTTP_MULTIPLEXING")? {
+            self.multiplexing = Some(multiplexing);
+        }
+        // https://doc.rust-lang.org/nightly/cargo/reference/config.html#httpuser-agent
+        if let Some(user_agent) = cx.env("CARGO_HTTP_USER_AGENT")? {
+            self.user_agent = Some(user_agent);
+        }
+        Ok(())
+    }
+}
+
 impl ApplyEnv for NetConfig {
     fn apply_env(&mut self, cx: &ResolveContext) -> Result<()> {
         // https://doc.rust-lang.org/nightly/cargo/reference/config.html#netretry
@@ -344,9 +403,9 @@ mod tests {
         let mut config = crate::de::BuildConfig::default();
         let cx =
             &ResolveOptions::default().env(env_list).into_context(std::env::current_dir().unwrap());
-        config.rustc_wrapper = Some(Value { val: "rustc_wrapper".to_string(), definition: None });
+        config.rustc_wrapper = Some(Value { val: "rustc_wrapper".to_owned(), definition: None });
         config.rustc_workspace_wrapper =
-            Some(Value { val: "rustc_workspace_wrapper".to_string(), definition: None });
+            Some(Value { val: "rustc_workspace_wrapper".to_owned(), definition: None });
         config.apply_env(cx).unwrap();
         assert!(config.rustc_wrapper.is_none());
         assert!(config.rustc_workspace_wrapper.is_none());
