@@ -39,6 +39,7 @@ See also the [`get` example](https://github.com/taiki-e/cargo-config2/blob/HEAD/
 #![forbid(unsafe_code)]
 #![warn(
     // Lints that may help when writing public library.
+    missing_debug_implementations,
     // missing_docs,
     clippy::alloc_instead_of_core,
     clippy::exhaustive_enums,
@@ -46,7 +47,7 @@ See also the [`get` example](https://github.com/taiki-e/cargo-config2/blob/HEAD/
     clippy::impl_trait_in_params,
     // clippy::missing_inline_in_public_items,
     // clippy::std_instead_of_alloc,
-    clippy::std_instead_of_core,
+    // clippy::std_instead_of_core,
 )]
 #![allow(clippy::must_use_candidate)]
 
@@ -75,7 +76,7 @@ mod value;
 mod walk;
 
 #[doc(no_inline)]
-pub use crate::de::{Color, Frequency, RegistriesProtocol, When};
+pub use crate::de::{Color, Frequency, RegistriesProtocol, VersionControlSoftware, When};
 pub use crate::{
     easy::{
         BuildConfig, Config, DocConfig, EnvConfigValue, Flags, FutureIncompatReportConfig,
@@ -83,6 +84,6 @@ pub use crate::{
         TermConfig, TermProgressConfig,
     },
     error::Error,
-    resolve::{ResolveOptions, TargetTriple, TargetTripleRef},
+    resolve::{CargoVersion, ResolveOptions, RustcVersion, TargetTriple, TargetTripleRef},
     walk::Walk,
 };
