@@ -3,13 +3,16 @@
 use std::collections::{btree_map, BTreeMap};
 
 use crate::{
-    de::{self, RegistriesProtocol},
+    de::{self, RegistriesProtocol, VersionControlSoftware},
     error::{Context as _, Result},
     value::Value,
     Color, Frequency, When,
 };
 
-// https://github.com/rust-lang/cargo/blob/0.67.0/src/cargo/util/config/mod.rs#L1900-L1908
+// https://github.com/rust-lang/cargo/blob/0.74.0/src/cargo/util/config/mod.rs#L2107-L2115
+// TODO:
+// - https://github.com/rust-lang/cargo/pull/12515
+// - https://github.com/rust-lang/cargo/blob/0.80.0/src/cargo/util/context/mod.rs#L2159-L2167
 //
 // > If `force` is true, primitive (non-container) types will override existing values.
 // > If false, the original will be kept and the new value ignored.
@@ -19,23 +22,23 @@ use crate::{
 // > Container and non-container types cannot be mixed.
 pub(crate) trait Merge {
     /// Merges given config into this config.
-    fn merge(&mut self, from: Self, force: bool) -> Result<()>;
+    fn merge(&mut self, low: Self, force: bool) -> Result<()>;
 }
 
 macro_rules! merge_non_container {
     ($($ty:tt)*) => {
         impl Merge for $($ty)* {
-            fn merge(&mut self, from: Self, force: bool) -> Result<()> {
+            fn merge(&mut self, low: Self, force: bool) -> Result<()> {
                 if force {
-                    *self = from;
+                    *self = low;
                 }
                 Ok(())
             }
         }
         impl Merge for Value<$($ty)*> {
-            fn merge(&mut self, from: Self, force: bool) -> Result<()> {
+            fn merge(&mut self, low: Self, force: bool) -> Result<()> {
                 if force {
-                    *self = from;
+                    *self = low;
                 }
                 Ok(())
             }
@@ -47,30 +50,34 @@ merge_non_container!(i32);
 merge_non_container!(u32);
 merge_non_container!(String);
 merge_non_container!(Color);
+merge_non_container!(VersionControlSoftware);
 merge_non_container!(Frequency);
 merge_non_container!(When);
 merge_non_container!(RegistriesProtocol);
 
 impl<T: Merge> Merge for Option<T> {
-    fn merge(&mut self, from: Self, force: bool) -> Result<()> {
-        match (self, from) {
+    fn merge(&mut self, low: Self, force: bool) -> Result<()> {
+        match (self, low) {
             (_, None) => {}
-            (this @ None, from) => *this = from,
-            (Some(this), Some(from)) => this.merge(from, force)?,
+            (this @ None, low) => *this = low,
+            (Some(this), Some(low)) => this.merge(low, force)?,
         }
         Ok(())
     }
 }
 impl Merge for de::StringOrArray {
-    fn merge(&mut self, from: Self, force: bool) -> Result<()> {
-        match (self, from) {
-            (this @ de::StringOrArray::String(_), from @ de::StringOrArray::String(_)) => {
+    fn merge(&mut self, low: Self, force: bool) -> Result<()> {
+        match (self, low) {
+            (this @ de::StringOrArray::String(_), low @ de::StringOrArray::String(_)) => {
                 if force {
-                    *this = from;
+                    *this = low;
                 }
             }
-            (de::StringOrArray::Array(this), de::StringOrArray::Array(mut from)) => {
-                this.append(&mut from);
+            (de::StringOrArray::Array(this), de::StringOrArray::Array(mut low)) => {
+                // https://doc.rust-lang.org/nightly/cargo/reference/config.html#hierarchical-structure
+                // > Arrays will be joined together with higher precedence items being placed later in the merged array.
+                low.append(this);
+                *this = low;
             }
             (expected, actual) => {
                 bail!("expected {}, but found {}", expected.kind(), actual.kind());
@@ -80,11 +87,11 @@ impl Merge for de::StringOrArray {
     }
 }
 impl Merge for de::PathAndArgs {
-    fn merge(&mut self, mut from: Self, force: bool) -> Result<()> {
-        match (self.deserialized_repr, from.deserialized_repr) {
+    fn merge(&mut self, mut low: Self, force: bool) -> Result<()> {
+        match (self.deserialized_repr, low.deserialized_repr) {
             (de::StringListDeserializedRepr::String, de::StringListDeserializedRepr::String) => {
                 if force {
-                    *self = from;
+                    *self = low;
                 }
             }
             (de::StringListDeserializedRepr::Array, de::StringListDeserializedRepr::Array) => {
@@ -98,8 +105,8 @@ impl Merge for de::PathAndArgs {
                 // # a/.cargo/config
                 // [doc]
                 // browser = ["a"]
-                self.args.push(from.path.0);
-                self.args.append(&mut from.args);
+                self.args.push(low.path.0);
+                self.args.append(&mut low.args);
             }
             (expected, actual) => {
                 bail!("expected {}, but found {}", expected.as_str(), actual.as_str());
@@ -109,15 +116,18 @@ impl Merge for de::PathAndArgs {
     }
 }
 impl Merge for de::StringList {
-    fn merge(&mut self, mut from: Self, force: bool) -> Result<()> {
-        match (self.deserialized_repr, from.deserialized_repr) {
+    fn merge(&mut self, mut low: Self, force: bool) -> Result<()> {
+        match (self.deserialized_repr, low.deserialized_repr) {
             (de::StringListDeserializedRepr::String, de::StringListDeserializedRepr::String) => {
                 if force {
-                    *self = from;
+                    *self = low;
                 }
             }
             (de::StringListDeserializedRepr::Array, de::StringListDeserializedRepr::Array) => {
-                self.list.append(&mut from.list);
+                // https://doc.rust-lang.org/nightly/cargo/reference/config.html#hierarchical-structure
+                // > Arrays will be joined together with higher precedence items being placed later in the merged array.
+                low.list.append(&mut self.list);
+                self.list = low.list;
             }
             (expected, actual) => {
                 bail!("expected {}, but found {}", expected.as_str(), actual.as_str());
@@ -127,20 +137,20 @@ impl Merge for de::StringList {
     }
 }
 impl Merge for de::EnvConfigValue {
-    fn merge(&mut self, from: Self, force: bool) -> Result<()> {
-        match (self, from) {
-            (Self::Value(this), Self::Value(from)) => {
+    fn merge(&mut self, low: Self, force: bool) -> Result<()> {
+        match (self, low) {
+            (Self::Value(this), Self::Value(low)) => {
                 if force {
-                    *this = from;
+                    *this = low;
                 }
             }
             (
                 Self::Table { value: this_value, force: this_force, relative: this_relative },
-                Self::Table { value: from_value, force: from_force, relative: from_relative },
+                Self::Table { value: low_value, force: low_force, relative: low_relative },
             ) => {
-                this_value.merge(from_value, force)?;
-                this_force.merge(from_force, force)?;
-                this_relative.merge(from_relative, force)?;
+                this_value.merge(low_value, force)?;
+                this_force.merge(low_force, force)?;
+                this_relative.merge(low_relative, force)?;
             }
             (expected, actual) => {
                 bail!("expected {}, but found {}", expected.kind(), actual.kind());
@@ -150,15 +160,18 @@ impl Merge for de::EnvConfigValue {
     }
 }
 impl Merge for de::Flags {
-    fn merge(&mut self, mut from: Self, force: bool) -> Result<()> {
-        match (self.deserialized_repr, from.deserialized_repr) {
+    fn merge(&mut self, mut low: Self, force: bool) -> Result<()> {
+        match (self.deserialized_repr, low.deserialized_repr) {
             (de::StringListDeserializedRepr::String, de::StringListDeserializedRepr::String) => {
                 if force {
-                    *self = from;
+                    *self = low;
                 }
             }
             (de::StringListDeserializedRepr::Array, de::StringListDeserializedRepr::Array) => {
-                self.flags.append(&mut from.flags);
+                // https://doc.rust-lang.org/nightly/cargo/reference/config.html#hierarchical-structure
+                // > Arrays will be joined together with higher precedence items being placed later in the merged array.
+                low.flags.append(&mut self.flags);
+                self.flags = low.flags;
             }
             (expected, actual) => {
                 bail!("expected {}, but found {}", expected.as_str(), actual.as_str());
@@ -168,8 +181,8 @@ impl Merge for de::Flags {
     }
 }
 impl<V: Merge + Clone + core::fmt::Debug> Merge for BTreeMap<String, V> {
-    fn merge(&mut self, from: Self, force: bool) -> Result<()> {
-        for (key, value) in from {
+    fn merge(&mut self, low: Self, force: bool) -> Result<()> {
+        for (key, value) in low {
             match self.entry(key.clone()) {
                 btree_map::Entry::Occupied(mut entry) => {
                     let entry = entry.get_mut();
